use actix_web::post;
use actix_web::{get, HttpRequest, HttpResponse, Responder};
use serde::Deserialize;

use crate::item;
use crate::routes::bad_req;

macro_rules! get_itemdb {
    ($req:expr) => {{
        let itemdb: &actix_web::web::Data<item::ItemDB> = $req.app_data().unwrap();
        itemdb
    }};
}

#[derive(Deserialize, Debug)]
pub struct SupplyForm {
    item: String,
    variant: String,
    amount: Option<usize>,
    price: String,
    origin: String,
}

#[post("/supply")]
pub async fn supply_route(
    req: HttpRequest,
    form: actix_web::web::Form<SupplyForm>,
) -> actix_web::Result<impl Responder> {
    let itemdb = get_itemdb!(req);
    println!("{form:?}");
    let variant = itemdb
        .get_item(&form.item)
        .ok_or_else(|| actix_web::error::ErrorBadRequest("The item does not exist"))?
        .variant(&form.variant)
        .await
        .ok_or_else(|| actix_web::error::ErrorBadRequest("The variant does not exist"))?;

    let transaction_id = variant
        .supply(
            form.amount.unwrap_or(1),
            form.price
                .clone()
                .try_into()
                .map_err(|_| bad_req("Price malformed"))?,
            &form.origin,
        )
        .await;
    Ok(actix_web::HttpResponse::Ok().json(serde_json::json!({"uuid": transaction_id})))
}

#[get("/items")]
pub async fn get_items_route(r: HttpRequest) -> impl Responder {
    let itemdb = get_itemdb!(r);
    let items = itemdb.items();
    actix_web::HttpResponse::Ok().json(serde_json::json!({"items": items}))
}

#[derive(Deserialize, Debug)]
pub struct AddVariantForm {
    pub variant: String,
    pub amount: usize,
}

#[post("/item/{item_id}/variant")]
pub async fn add_variant_route(
    r: HttpRequest,
    f: actix_web::web::Form<AddVariantForm>,
) -> actix_web::Result<impl Responder> {
    let id = r.match_info().query("item_id");
    let itemdb = get_itemdb!(r);
    let item = itemdb
        .get_item(id)
        .ok_or_else(|| bad_req("The item does not exist"))?;
    item.add_variant(&f.variant, f.amount).await;
    Ok(HttpResponse::Ok())
}

#[get("/item/{item_id}/variants")]
pub async fn item_variants_page(r: HttpRequest) -> actix_web::Result<impl Responder> {
    let id = r.match_info().query("item_id");
    let itemdb = get_itemdb!(r);
    let item = itemdb
        .get_item(id)
        .ok_or_else(|| bad_req("The item does not exist"))?;
    let variants = item.get_variants().await;

    Ok(HttpResponse::Ok().json(serde_json::json!({
        "item": id,
        "variants": variants
    })))
}
